package net.moonleay.mods.gimble.client.editor

import net.minecraft.client.MinecraftClient
import net.moonleay.mods.gimble.client.util.ChatUtil
import net.moonleay.mods.gimble.client.editor.modes.Mode
import net.moonleay.mods.gimble.client.editor.modes.ModeModifier

object Editor {
    var CURRENT_MODE = Mode.NORMAL
    val CURRENT_MODE_MODIFIER = mutableListOf<ModeModifier>()
    private val TEMP_DISABLED_MODIFIERS = mutableListOf<ModeModifier>()


    fun toggleModifier(mod: ModeModifier) {
        if (CURRENT_MODE.incompatibleModifiers.contains(mod)){
            if (TEMP_DISABLED_MODIFIERS.contains(mod))
                TEMP_DISABLED_MODIFIERS.remove(mod)
            else
                TEMP_DISABLED_MODIFIERS.add(mod)
        }
        else {
            if (CURRENT_MODE_MODIFIER.contains(mod))
                CURRENT_MODE_MODIFIER.remove(mod)
            else
                CURRENT_MODE_MODIFIER.add(mod)
        }

        this.onModifiersUpdated()
    }

    fun onModifiersUpdated() {
        CURRENT_MODE_MODIFIER.sort()
        this.checkForIncompatibleModeModifiers()
    }

    fun checkForIncompatibleModeModifiers() {
        if (TEMP_DISABLED_MODIFIERS.size > 0) {
            CURRENT_MODE_MODIFIER.addAll(TEMP_DISABLED_MODIFIERS)
            TEMP_DISABLED_MODIFIERS.clear()
        }

        val incompatibleModeModifiers = CURRENT_MODE.incompatibleModifiers
        for (mod in CURRENT_MODE_MODIFIER) {
            if (incompatibleModeModifiers.contains(mod)) {
                TEMP_DISABLED_MODIFIERS.add(mod)
            }
        }
        for (mod in TEMP_DISABLED_MODIFIERS) {
            if (CURRENT_MODE_MODIFIER.contains(mod)) { // This check is probably useless.
                CURRENT_MODE_MODIFIER.remove(mod)
            }
        }

        if(TEMP_DISABLED_MODIFIERS.isNotEmpty()) {
        ChatUtil.addToChatHistory("The following modifiers are not supported by this editor mode and are therefore currently disabled: " +
                this.getListAsString(TEMP_DISABLED_MODIFIERS), MinecraftClient.getInstance())
        }
    }

    private fun getListAsString(list: List<ModeModifier>): String{
        if(list.isEmpty())
            return "Empty list"
        val sb = StringBuilder()
        for (mod in list) {
            sb.append(mod.displayName)
            sb.append(" ")
        }
        return sb.toString().dropLast(1)
    }

    fun getCurrentMode(): Mode {
        return CURRENT_MODE
    }

    fun getCurrentModifier(): List<ModeModifier> {
        return CURRENT_MODE_MODIFIER
    }
}
