package net.moonleay.mods.gimble.mixin;

import net.minecraft.client.MinecraftClient;
import net.minecraft.client.font.TextRenderer;
import net.minecraft.client.gui.hud.InGameHud;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.text.Text;
import net.moonleay.mods.gimble.client.editor.Editor;
import net.moonleay.mods.gimble.client.editor.modes.Mode;
import net.moonleay.mods.gimble.client.editor.modes.ModeModifier;
import org.jetbrains.annotations.NotNull;
import org.spongepowered.asm.mixin.Final;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import java.util.List;

@Mixin(InGameHud.class)
public class HudMixin {

    @Shadow @Final private MinecraftClient client;

    @Inject(method = "renderStatusEffectOverlay", at = @At("HEAD"))
    private void render(MatrixStack matrices, CallbackInfo ci) {
        Mode currentMode = Editor.INSTANCE.getCurrentMode();
        StringBuilder displayText = getStringBuilder(currentMode);

        MinecraftClient mc = MinecraftClient.getInstance();
        TextRenderer tr = mc.textRenderer;
//        int screenWidth = mc.getWindow().getWidth();
//        int screenHeight = mc.getWindow().getHeight();
//        int scale = (mc.options.getGuiScale().getValue() == 0 ? 1 : mc.options.getGuiScale().getValue());
//        int bottomY = screenHeight / scale - 4 - tr.fontHeight - 12;

        tr.drawWithShadow(matrices,
                Text.of(displayText.toString()),
                4, 4,
                currentMode.getColor());
    }

    @Unique
    private static @NotNull StringBuilder getStringBuilder(Mode currentMode) {
        List<ModeModifier> currentModifier = Editor.INSTANCE.getCurrentModifier();
        StringBuilder displayText = new StringBuilder(currentMode.getDisplayName());
        if (!currentModifier.isEmpty()) {
            displayText.append(" [");
            for (int i = 0; i < currentModifier.size(); i++) {
                displayText.append(currentModifier.get(i).getDisplayName());
                if (i != currentModifier.size() - 1) {
                    displayText.append(", ");
                }
            }
            displayText.append("]");
        }
        return displayText;
    }

    @Unique
    private int getXRight(String txt, int screenWidth) {
        return screenWidth / (MinecraftClient.getInstance().options.getGuiScale().getValue() == 0 ? 1 : MinecraftClient.getInstance().options.getGuiScale().getValue()) - 4 - MinecraftClient.getInstance().textRenderer.getWidth(Text.of(txt));
    }
}
